@@ -26,7 +26,7 @@ is.installed <- function(pkg) is.element(pkg, utils::installed.packages()[,1])
 # generic FFT functions
 FFT <- function(X,inverse=FALSE)
 {
-  if(is.null(ncol(X)))
+  if(is.null(ncol(X)) || is.na(ncol(X)))
   { 
     if(!inverse) { X <- stats::fft(X) }
     else { X <- stats::fft(X,inverse=TRUE)/length(X) }
@@ -43,7 +43,7 @@ FFT <- function(X,inverse=FALSE)
 # fastest FFT functions... don't use on integers
 FFTW <- function(X,inverse=FALSE)
 {
-  if(is.null(ncol(X)))
+  if(is.null(ncol(X)) || is.na(ncol(X)))
   { 
     if(!inverse) { X <- fftw::FFT(X) }
     else { X <- fftw::IFFT(X) }
@@ -113,7 +113,6 @@ plot.list <- function(x,...)
 }
 #methods::setMethod("plot",signature(x="list"), function(x,...) plot.list(x,...))
 
-
 # forwarding function for list of a particular datatype
 summary.list <- function(object,...)
 {
@@ -160,7 +159,7 @@ Adj <- function(M) { t(Conj(M)) }
 He <- function(M) { (M + Adj(M))/2 }
 
 # Positive definite solver
-PDsolve <- function(M)
+PDsolve <- function(M,pseudo=FALSE)
 {
   # symmetrize
   M <- He(M)
@@ -172,7 +171,20 @@ PDsolve <- function(M)
   
   # now a correlation matrix that is easy to invert
   M <- M/W
-  M <- qr.solve(M)
+  if(!pseudo)
+  { M <- qr.solve(M) }
+  else
+  {
+    M <- eigen(M)
+    V <- M$vectors
+    M <- M$values
+    INDEX <- (abs(M) >= length(M)*.Machine$double.eps) # conventional tolerance
+    M[INDEX] <- 1/M[INDEX]
+    INDEX <- !INDEX
+    M[INDEX] <- 0
+    M <- lapply(1:length(M),function(i) M[i]*(V[,i] %o% Conj(V[,i])) )
+    M <- Reduce('+',M)
+  }
   M <- M/W
 
   # symmetrize
@@ -181,6 +193,14 @@ PDsolve <- function(M)
   return(M)
 }
 
+# condition number
+conditionNumber <- function(M)
+{
+  M <- eigen(M)$values
+  M <- range(M)
+  return(M[2]/M[1])
+}
+
 # sqrtm fails on 1x1 matrix
 # I cannot figure out how to make this "Note" go away!
 # x <- Matrix::Matrix(x,sparse=FALSE,doDiag=FALSE)
